/* Inclusions */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "random.h"

/* Constants */
#define SIZE 4096
#define ABS(a) ((a)>0?(a):-1*(a))

int intrand(int hi)
/* Function returning a random integer between 0 and hi-1 */
{
  return itrunc(hi*ranmar());
}

main(int argc, char **argv)
{
  int seed1, seed2;
  double cutoff, avalcutoff;
  int mutsite, oldmutsite, mutdiff;
  double barrier[SIZE];
  double fitness[SIZE];
  char infile[256], outfile[256];
  int useinfile=0;
  int diff[SIZE];
  int avals[25*SIZE];
  int avalsize=0;
  int equil, run;
  int i,j, mplus1, mminus1;
  double min;
  int minsite;
  int dsize;
  double add;
  FILE *fp;


#ifdef BAK
  fprintf(stderr,"\nBak Version.\n\n");
#else  
  fprintf(stderr,"\nMy Version.\n\n");
#endif
  fflush(stderr);

  fprintf(stderr,"Input seed1: ");
  fscanf(stdin,"%d",&seed1);
  fprintf(stderr,"Input seed2: ");
  fscanf(stdin,"%d",&seed2);
  fprintf(stderr,"Input mutation cutoff: ");
  fscanf(stdin,"%lf",&cutoff);
  fprintf(stderr,"Input avalanche cutoff ( < 0 for no avalanches): ");
  fscanf(stdin,"%lf",&avalcutoff);  
  fprintf(stderr,"Input equilibration time: ");
  fscanf(stdin,"%d",&equil);
  fprintf(stderr,"Input running time: ");
  fscanf(stdin,"%d",&run);
  fprintf(stderr,"Use input file (1=Yes, 0=No)? ");
  fscanf(stdin,"%d",&useinfile);
  if(useinfile == 1) {
    fprintf(stderr,"Input file name: ");
    fscanf(stdin,"%s",infile);
  }

/*  fprintf(stderr,"avalcutoff = %f\n",avalcutoff); */

  rmarin(seed1, seed2);

  if(useinfile == 0) {
    for(i=0;i<SIZE;i++) fitness[i] = ranmar();
    for(i=0;i<SIZE;i++) barrier[i] = realrand(fitness[i],1);    
  } else {
    fp = fopen(infile,"r");
    for(i=0;i<SIZE;i++) fscanf(fp,"%lf %lf",&barrier[i],&fitness[i]);
    fclose(fp);
  }

  for(i=0;i<SIZE;i++) {
    diff[i] = 0;
  }

  for(i=0;i<5*SIZE;i++) {
    avals[i] = 0;
  }

/* Equilibrate */
  for(i=0;i<equil;i++) {
    for(j=0;j<SIZE;j++) {
      if(j==0) {
	min = barrier[0];
	minsite = 0;
      } else {
	if(barrier[j]<min) {
	  min = barrier[j];
	  minsite = j;
	}
      }
    }

    if(minsite != SIZE-1) mplus1 = minsite + 1;
    else mplus1 = 0;
    if(minsite != 0) mminus1 = minsite - 1;
    else mminus1 = SIZE-1;

    barrier[minsite] = ranmar();
    fitness[minsite] = realrand(fitness[minsite]-barrier[minsite],1.0);
#ifdef BAK
    barrier[mplus1] = ranmar();
    fitness[mplus1] = realrand(fitness[mplus1]-barrier[mplus1],1.0);
    barrier[mminus1] = ranmar(); 
    fitness[mminus1] = realrand(fitness[mminus1]-barrier[mminus1],1.0);
#else
    add = (2*ranmar()-1)*cutoff;
    if(barrier[mplus1]+add < 0) {
      barrier[mplus1] = 0.0;
    } else { 
      if(barrier[mplus1]+add > 1.0) {
	barrier[mplus1] = 1.0;
      } else {
	barrier[mplus1] += add;
      }
    }
    add = (2*ranmar()-1)*cutoff;
    if(barrier[mminus1]+add < 0) {
      barrier[mminus1] = 0.0;
    } else { 
      if(barrier[mminus1]+add > 1.0) {
	barrier[mminus1] = 1.0;
      } else {
	barrier[mminus1] += add;
      }
    }
#endif

  } /* End of equilibration */


/* Do the runs */

  for(i=0;i<run;i++) {
    for(j=0;j<SIZE;j++) {
      if(j == 0) {
	min = barrier[0];
	minsite = 0;
      } else {
	if(barrier[j]<min) {
	  min = barrier[j];
	  minsite = j;
	}
      }
    }

/*    fprintf(stderr,"min = %f\n",min); */

    if(minsite != SIZE-1) mplus1 = minsite + 1;
    else mplus1 = 0;
    if(minsite != 0) mminus1 = minsite - 1;
    else mminus1 = SIZE-1;

    barrier[minsite] = ranmar();
    fitness[minsite] = realrand(fitness[minsite]-barrier[minsite],1.0);
#ifdef BAK
    barrier[mplus1] = ranmar();
    fitness[mplus1] = realrand(fitness[mplus1]-barrier[mplus1],1.0);
    barrier[mminus1] = ranmar(); 
    fitness[mminus1] = realrand(fitness[mminus1]-barrier[mminus1],1.0);
#else
    add = (2*ranmar()-1)*cutoff;
    if(barrier[mplus1]+add < 0) {
      barrier[mplus1] = 0.0;
    } else { 
      if(barrier[mplus1]+add > 1) {
	barrier[mplus1] = 1.0;
      } else {
	barrier[mplus1] += add;
      }
    }
    add = (2*ranmar()-1)*cutoff;
    if(barrier[mminus1]+add < 0) {
      barrier[mminus1] = 0.0;
    } else { 
      if(barrier[mminus1]+add > 1) {
	barrier[mminus1] = 1.0;
      } else {
	barrier[mminus1] += add;
      }
    }
#endif

/* Check the avalanche status here */    
/*    fprintf(stderr,"minsite = %f\n",barrier[minsite]); */
    if(min < avalcutoff) {
      avalsize++;
    } else {
      if(avalsize >= 25*SIZE) {
	fprintf(stderr,"Warning: avalanche overflow.\n");
	avalsize = 0;
      } else {
	avals[avalsize] += 1;
	avalsize = 0;
      }
    }

    
/* Collect data here */
/*    if(i!=0) {
      mutdiff = ABS(oldmutsite-minsite);
      if(mutdiff>SIZE/2) mutdiff = SIZE-mutdiff;
      diff[mutdiff] += 1;
    }
    oldmutsite = minsite;
*/

  }

/*  avals[avalsize] += 1; */

/* Write out the barriers */
  for(i=0;i<SIZE;i++) {
    fprintf(stdout,"%f %f\n",barrier[i], fitness[i]);
  }

/*
#ifdef BAK
  fp = fopen("diff.out.bak","w");
#else
  fp = fopen("diff.out.mine","w");
#endif
  for(i=1;i<=SIZE/2;i++) {
    fprintf(fp,"%d %d\n",i,diff[i]);
  }
  fclose(fp);
*/

#ifdef BAK
  fp = fopen("aval.out.bak","w");
#else
  fp = fopen("aval.out.mine","w");
#endif
  for(i=1;i<25*SIZE;i++) {
    fprintf(fp,"%d %d\n",i,avals[i]);
  }
  fclose(fp);

}





